/* 
 * Copyright (C) 2014 Vincent Vansuyt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import ij.gui.GenericDialog;
import ij.gui.ImageWindow;

/**
 * @description
 * English description :
 * Convert a Lab stack image to a new RGB image  
 *
 * French description :
 * Convertit une image-stack contenant 3 slices L,a,b en une nouvelle image RGB
  
 * @version v1.00
 * @author Vincent Vansuyt
 */
public class Convert_Lab_stack_to_RGB_image implements PlugIn {

	
	/**
	 * Point d'entree du plugin
	 *
	 * @param string
	 */
	@Override
	public void run(String string) {
		IJ.log( "\\Clear" );
		IJ.log( "------- Convert a Lab stack to a new RGB image - begin ------" );
		IJ.log( "Dossier log : " + System.getProperty( "user.dir" ) );
		//imp IJ.getImage();
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp == null ) {
			IJ.noImage () ;
			return ;
		}				
		
		// − doit être stack		
		int nNb_slices = imp.getNSlices();
		String strM = "Nb of slices = " + nNb_slices;
		IJ.log( strM );		
		int nWidth = imp.getWidth();
		int nHeight = imp.getHeight();
		strM = "Image w:" + nWidth + " x h:" + nHeight;
		IJ.log( strM );
		if ( nNb_slices != 3 ) {
			strM = "Image must contain 3 slices (L, a, b)";
			IJ.showMessage( "Warning", strM );
			IJ.log( strM );
			return ;
		}
		String strNom_img_src = imp.getTitle();

		ImageStack imgstack = imp.getImageStack();
		ImageProcessor ip_L = imgstack.getProcessor( 0 + 1 );
		ImageProcessor ip_a = (imgstack.getProcessor( 1 + 1 ));
		ImageProcessor ip_b = (imgstack.getProcessor( 2 + 1 ));
		
		// Création de l'image de résultat RGB
		final int ncNbSclices = 1;
		// "RGB Black" "8− bit White"
		String strNom_img_dest = strNom_img_src + "_toRGB";
		ImagePlus imp_result = IJ.createImage( strNom_img_dest, "RGB Black" , nWidth , nHeight , ncNbSclices ) ;
		ImageProcessor ip_result = imp_result.getProcessor();		
		double[] clLab = {0.0, 0.0, 0.0 };
		int[] clRGB = {0,0,0};
		Colors fxCouleurs = new Colors();
		IJ.log( "Parsing image..." );
		for ( int y = 0; y < nHeight ; ++y ) {
			IJ.showProgress( y, nHeight );
			for ( int x = 0; x < nWidth ; ++x ) {
				clLab[0] = ip_L.getf( x, y );
				clLab[1] = ip_a.getf( x, y );
				clLab[2] = ip_b.getf( x, y );
				clRGB = fxCouleurs.convertLab_to_RGB( clLab );
				ip_result.putPixel( x, y, clRGB );
			}			
		}
		imp_result.show();
		/*
		IJ.log( "Color tests" );
		int[][] tabCl_RGB = {{255,0,0},{0,255,0},{0,0,255}, {255,255,0}};
		for ( int i = 0; i < tabCl_RGB.length; ++i ) {
			clRGB[0] = tabCl_RGB[i][0];
			clRGB[1] = tabCl_RGB[i][1];
			clRGB[2] = tabCl_RGB[i][2];
			int nNum_couleur = i + 1;
			IJ.log( "---------- Color : " + nNum_couleur + "/" + tabCl_RGB.length +" ---------------");
			IJ.log( "Couleur RGB(" + clRGB[0] + ", " + clRGB[1] + ", " + clRGB[2] + " )" ) ;
			clLab = fxCouleurs.convertRGB_to_Lab( clRGB );
			IJ.log( "Conversion RGB vers Lab(" + String.format("%.3f", clLab[0] ) +  ", " + String.format("%.3f", clLab[1] ) + ", " + String.format("%.3f", clLab[2] ) + " )" ) ;
			clRGB = fxCouleurs.convertLab_to_RGB( clLab );
			IJ.log( "Conversion Lab vers RGB(" + clRGB[0] + ", " + clRGB[1] + ", " + clRGB[2] + " )" ) ;			
		}*/
		IJ.log( "------- Convert a Lab stack to a new RGB image - end ------" );
	}// run
	
}// Convert_Lab_stack_to_RGB_image
