/* 
 * Copyright (C) 2014 Vincent Vansuyt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import ij.gui.GenericDialog;
import ij.gui.ImageWindow;
		
/**
 * @description
 * English description :
 * Convert a RGB image to a new image-stack containing 3 slices L,a,b
 *
 * French description :
 * Convertit image RGB en une nouvelle image-stack contenant 3 slices L,a,b 
  
 * @version v1.00
 * @author Vincent Vansuyt
 */
public class Convert_RGB_image_to_Lab_stack implements PlugIn {

	
	/**
	 * Point d'entree du plugin
	 *
	 * @param string
	 */
	@Override
	public void run(String string) {
		IJ.log( "\\Clear" );
		IJ.log( "------- Convert a RGB image to a new image-stack Lab - begin ------" );
		IJ.log( "Log folder : " + System.getProperty( "user.dir" ) );
		//imp IJ.getImage();
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp == null ) {
			IJ.noImage () ;
			return ;
		}			
		
		Colors fxCouleurs = new Colors();
		
		// − doit être RGB 8 ou 16 bits
		int nNb_slices = imp.getNSlices();
		String strM = "Nb of slices = " + nNb_slices;
		IJ.log( strM );		
		int nWidth = imp.getWidth();
		int nHeight = imp.getHeight();
		strM = "Image w:" + nWidth + " x h:" + nHeight;
		IJ.log( strM );
		int nType = imp.getType () ;
		if ( nType != ImagePlus.COLOR_RGB ) {
			String strType = fxCouleurs.getImage_type( nType );
			strM = "Image must be RGB type (actual type : " + strType + ")" ;
			IJ.showMessage( "Warning", strM );
			IJ.log( strM );
			IJ.log( "------- Convert a RGB image to a new image-stack Lab - fin ------" );
			return ;
		}		
		String strNom_img_src = imp.getTitle();

		ImageProcessor ipRGB = imp.getProcessor();
		
		// Création de l'image de résultat RGB
		final int ncNb_sclices = 3;		
		// "RGB Black" "8− bit White"
		String strNom_img_dest = strNom_img_src + "_toLab";
		ImagePlus imp_result = IJ.createImage( strNom_img_dest, "32-bit black" , nWidth , nHeight , ncNb_sclices ) ;
		ImageStack imgstack = imp_result.getImageStack();
		ImageProcessor ip_L = imgstack.getProcessor( 0 + 1 );
		ImageProcessor ip_a = imgstack.getProcessor( 1 + 1 );
		ImageProcessor ip_b = imgstack.getProcessor( 2 + 1 );		
		
		double[] clLab = {0.0, 0.0, 0.0 };
		int[] clRGB = {0,0,0,0};
		
		IJ.log( "Parsing image..." );
		// Initialisation des mini-maxi
		clRGB = ipRGB.getPixel( 0, 0, clRGB );
		clLab = fxCouleurs.convertRGB_to_Lab( clRGB );
		double nCl_min = clLab[0];
		double nCl_max = clLab[0];
		// Parcours de l'image et calcul des composantes Lab
		for ( int y = 0; y < nHeight ; ++y ) {
			IJ.showProgress( y, nHeight );
			for ( int x = 0; x < nWidth ; ++x ) {
				clRGB = ipRGB.getPixel( x, y, clRGB );
				clLab = fxCouleurs.convertRGB_to_Lab( clRGB );
				ip_L.putPixelValue( x, y, clLab[0] );
				ip_a.putPixelValue( x, y, clLab[1] );
				ip_b.putPixelValue( x, y, clLab[2] );
				for ( int i = 0; i < 2; ++i ) {
					nCl_min = Math.min( nCl_min, clLab[i] );
					nCl_max = Math.max( nCl_max, clLab[i] );
				}
			}			
		}	
		//IJ.log( "---------- Gray min : " + String.format( "%.2f", nCl_min ) + ", max : " + String.format("%.2f", nCl_max ));
		//imp_result.getProcessor().resetMinAndMax();
		imp_result.getProcessor().setMinAndMax( nCl_min, nCl_max );
		imp_result.show();
		
		/*IJ.log( "Color tests" );
		int[][] tabCl_RGB = {{255,0,0},{0,255,0},{0,0,255}, {255,255,0}};
		for ( int i = 0; i < tabCl_RGB.length; ++i ) {
			clRGB[0] = tabCl_RGB[i][0];
			clRGB[1] = tabCl_RGB[i][1];
			clRGB[2] = tabCl_RGB[i][2];
			int nNum_couleur = i + 1;
			IJ.log( "---------- Color : " + nNum_couleur + "/" + tabCl_RGB.length +" ---------------");
			IJ.log( "Couleur RGB(" + clRGB[0] + ", " + clRGB[1] + ", " + clRGB[2] + " )" ) ;
			clLab = fxCouleurs.convertRGB_to_Lab( clRGB );
			IJ.log( "Conversion RGB vers Lab(" + String.format("%.3f", clLab[0] ) +  ", " + String.format("%.3f", clLab[1] ) + ", " + String.format("%.3f", clLab[2] ) + " )" ) ;
			clRGB = fxCouleurs.convertLab_to_RGB( clLab );
			IJ.log( "Conversion Lab vers RGB(" + clRGB[0] + ", " + clRGB[1] + ", " + clRGB[2] + " )" ) ;			
		}*/
		IJ.log( "------- Convert a RGB image to a new image-stack Lab - end ------" );
	}// run
	
}// Convert_RGB_image_to_Lab_stack
